{% extends "oauth2_provider/base.html" %}

{% load i18n %}

{% block title %}
Success code={{code}}
{% endblock %}

{% block content %}
    <div class="block-center">
        {% if not error %}
            <h2>{% trans "Success" %}</h2>

            <p>{% trans "Please return to your application and enter this code:" %}</p>

            <p><code>{{ code }}</code></p>

        {% else %}
            <h2>Error: {{ error.error }}</h2>
            <p>{{ error.description }}</p>
        {% endif %}
    </div>
{% endblock %}
